@@ -1,4 +1,4 @@
-/*	$OpenBSD: json.c,v 1.7 2023/04/26 21:17:24 claudio Exp $ */
+/*	$OpenBSD: json.c,v 1.10 2023/06/22 09:07:04 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -16,10 +16,12 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <ctype.h>
 #include <err.h>
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "json.h"
@@ -36,6 +38,7 @@ enum json_type {
 static struct json_stack {
 	const char	*name;
 	unsigned int	count;
+	int		compact;
 	enum json_type	type;
 } stack[JSON_MAX_STACK];
 
@@ -47,9 +50,18 @@ static FILE *jsonfh;
 static void
 do_comma_indent(void)
 {
-	if (stack[level].count++ > 0)
+	char sp = '\n';
+
+	if (stack[level].compact)
+		sp = ' ';
+
+	if (stack[level].count++ > 0) {
 		if (!eb)
-			eb = fprintf(jsonfh, ",\n") < 0;
+			eb = fprintf(jsonfh, ",%c", sp) < 0;
+	}
+
+	if (stack[level].compact)
+		return;
 	if (!eb)
 		eb = fprintf(jsonfh, "\t%.*s", level, indent) < 0;
 }
@@ -105,6 +117,7 @@ void
 json_do_array(const char *name)
 {
 	int i, l;
+	char sp = '\n';
 
 	if ((l = do_find(ARRAY, name)) > 0) {
 		/* array already in use, close element and move on */
@@ -116,10 +129,12 @@ json_do_array(const char *name)
 	if (stack[level].type == ARRAY)
 		json_do_end();
 
+	if (stack[level].compact)
+		sp = ' ';
 	do_comma_indent();
 	do_name(name);
 	if (!eb)
-		eb = fprintf(jsonfh, "[\n") < 0;
+		eb = fprintf(jsonfh, "[%c", sp) < 0;
 
 	if (++level >= JSON_MAX_STACK)
 		errx(1, "json stack too deep");
@@ -127,12 +142,15 @@ json_do_array(const char *name)
 	stack[level].name = name;
 	stack[level].type = ARRAY;
 	stack[level].count = 0;
+	/* inherit compact setting from above level */
+	stack[level].compact = stack[level - 1].compact;
 }
 
 void
-json_do_object(const char *name)
+json_do_object(const char *name, int compact)
 {
 	int i, l;
+	char sp = '\n';
 
 	if ((l = do_find(OBJECT, name)) > 0) {
 		/* roll back to that object and close it */
@@ -140,10 +158,12 @@ json_do_object(const char *name)
 			json_do_end();
 	}
 
+	if (compact)
+		sp = ' ';
 	do_comma_indent();
 	do_name(name);
 	if (!eb)
-		eb = fprintf(jsonfh, "{\n") < 0;
+		eb = fprintf(jsonfh, "{%c", sp) < 0;
 
 	if (++level >= JSON_MAX_STACK)
 		errx(1, "json stack too deep");
@@ -151,23 +171,33 @@ json_do_object(const char *name)
 	stack[level].name = name;
 	stack[level].type = OBJECT;
 	stack[level].count = 0;
+	stack[level].compact = compact;
 }
 
 void
 json_do_end(void)
 {
-	if (stack[level].type == ARRAY) {
-		if (!eb)
-			eb = fprintf(jsonfh, "\n%.*s]", level, indent) < 0;
-	} else if (stack[level].type == OBJECT) {
-		if (!eb)
-			eb = fprintf(jsonfh, "\n%.*s}", level, indent) < 0;
-	} else {
+	char c;
+
+	if (stack[level].type == ARRAY)
+		c = ']';
+	else if (stack[level].type == OBJECT)
+		c = '}';
+	else
 		errx(1, "json bad stack state");
+
+	if (!stack[level].compact) {
+		if (!eb)
+			eb = fprintf(jsonfh, "\n%.*s%c", level, indent, c) < 0;
+	} else {
+		if (!eb)
+			eb = fprintf(jsonfh, " %c", c) < 0;
 	}
+
 	stack[level].name = NULL;
 	stack[level].type = NONE;
 	stack[level].count = 0;
+	stack[level].compact = 0;
 
 	if (level-- <= 0)
 		errx(1, "json stack underflow");
@@ -179,16 +209,58 @@ void
 json_do_printf(const char *name, const char *fmt, ...)
 {
 	va_list ap;
+	char *str;
+
+	va_start(ap, fmt);
+	if (!eb) {
+		if (vasprintf(&str, fmt, ap) == -1)
+			errx(1, "json printf failed");
+		json_do_string(name, str);
+		free(str);
+	}
+	va_end(ap);
+}
+
+void
+json_do_string(const char *name, const char *v)
+{
+	unsigned char c;
 
 	do_comma_indent();
-
 	do_name(name);
 	if (!eb)
 		eb = fprintf(jsonfh, "\"") < 0;
-	va_start(ap, fmt);
-	if (!eb)
-		eb = vfprintf(jsonfh, fmt, ap) < 0;
-	va_end(ap);
+	while ((c = *v++) != '\0' && !eb) {
+		/* skip escaping '/' since our use case does not require it */
+		switch (c) {
+		case '"':
+			eb = fprintf(jsonfh, "\\\"") < 0;
+			break;
+		case '\\':
+			eb = fprintf(jsonfh, "\\\\") < 0;
+			break;
+		case '\b':
+			eb = fprintf(jsonfh, "\\b") < 0;
+			break;
+		case '\f':
+			eb = fprintf(jsonfh, "\\f") < 0;
+			break;
+		case '\n':
+			eb = fprintf(jsonfh, "\\n") < 0;
+			break;
+		case '\r':
+			eb = fprintf(jsonfh, "\\r") < 0;
+			break;
+		case '\t':
+			eb = fprintf(jsonfh, "\\t") < 0;
+			break;
+		default:
+			if (iscntrl(c))
+				errx(1, "bad control character in string");
+			eb = putc(c, jsonfh) == EOF;
+			break;
+		}
+	}
 	if (!eb)
 		eb = fprintf(jsonfh, "\"") < 0;
 }
