@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.c,v 1.267 2024/09/04 15:06:36 claudio Exp $ */
+/*	$OpenBSD: bgpd.c,v 1.280 2024/12/03 13:46:53 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -53,8 +53,9 @@ int		control_setup(struct bgpd_config *);
 static void	getsockpair(int [2]);
 int		imsg_send_sockets(struct imsgbuf *, struct imsgbuf *,
 		    struct imsgbuf *);
-void		bgpd_rtr_connect(struct rtr_config *);
-void		bgpd_rtr_connect_done(int, struct bgpd_config *);
+void		bgpd_rtr_conn_setup(struct rtr_config *);
+void		bgpd_rtr_conn_setup_done(int, struct bgpd_config *);
+void		bgpd_rtr_conn_teardown(uint32_t);
 
 int			 cflags;
 volatile sig_atomic_t	 mrtdump;
@@ -71,12 +72,15 @@ char			*rcname;
 
 struct connect_elm {
 	TAILQ_ENTRY(connect_elm)	entry;
+	struct auth_state		auth_state;
 	uint32_t			id;
 	int				fd;
 };
 
 TAILQ_HEAD(, connect_elm)	connect_queue = \
-				    TAILQ_HEAD_INITIALIZER(connect_queue);
+				    TAILQ_HEAD_INITIALIZER(connect_queue),
+				socket_queue = \
+				    TAILQ_HEAD_INITIALIZER(socket_queue);
 u_int				connect_cnt;
 #define MAX_CONNECT_CNT		32
 
@@ -272,9 +276,16 @@ main(int argc, char *argv[])
 	    (ibuf_rde = malloc(sizeof(struct imsgbuf))) == NULL ||
 	    (ibuf_rtr = malloc(sizeof(struct imsgbuf))) == NULL)
 		fatal(NULL);
-	imsg_init(ibuf_se, pipe_m2s[0]);
-	imsg_init(ibuf_rde, pipe_m2r[0]);
-	imsg_init(ibuf_rtr, pipe_m2roa[0]);
+	if (imsgbuf_init(ibuf_se, pipe_m2s[0]) == -1 ||
+	    imsgbuf_set_maxsize(ibuf_se, MAX_BGPD_IMSGSIZE) == -1 ||
+	    imsgbuf_init(ibuf_rde, pipe_m2r[0]) == -1 ||
+	    imsgbuf_set_maxsize(ibuf_rde, MAX_BGPD_IMSGSIZE) == -1 ||
+	    imsgbuf_init(ibuf_rtr, pipe_m2roa[0]) == -1 ||
+	    imsgbuf_set_maxsize(ibuf_rtr, MAX_BGPD_IMSGSIZE) == -1)
+		fatal(NULL);
+	imsgbuf_allow_fdpass(ibuf_se);
+	imsgbuf_allow_fdpass(ibuf_rde);
+	imsgbuf_allow_fdpass(ibuf_rtr);
 	mrt_init(ibuf_rde, ibuf_se);
 	if (kr_init(&rfd, conf->fib_priority) == -1)
 		quit = 1;
@@ -358,7 +369,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 
 		if (handle_pollfd(&pfd[PFD_PIPE_SESSION], ibuf_se) == -1) {
 			log_warnx("main: Lost connection to SE");
-			msgbuf_clear(&ibuf_se->w);
+			imsgbuf_clear(ibuf_se);
 			free(ibuf_se);
 			ibuf_se = NULL;
 			quit = 1;
@@ -370,7 +381,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 
 		if (handle_pollfd(&pfd[PFD_PIPE_RDE], ibuf_rde) == -1) {
 			log_warnx("main: Lost connection to RDE");
-			msgbuf_clear(&ibuf_rde->w);
+			imsgbuf_clear(ibuf_rde);
 			free(ibuf_rde);
 			ibuf_rde = NULL;
 			quit = 1;
@@ -381,7 +392,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 
 		if (handle_pollfd(&pfd[PFD_PIPE_RTR], ibuf_rtr) == -1) {
 			log_warnx("main: Lost connection to RTR");
-			msgbuf_clear(&ibuf_rtr->w);
+			imsgbuf_clear(ibuf_rtr);
 			free(ibuf_rtr);
 			ibuf_rtr = NULL;
 			quit = 1;
@@ -404,7 +415,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 
 		for (i = PFD_CONNECT_START; i < npfd; i++)
 			if (pfd[i].revents != 0)
-				bgpd_rtr_connect_done(pfd[i].fd, conf);
+				bgpd_rtr_conn_setup_done(pfd[i].fd, conf);
 
  next_loop:
 		if (reconfig) {
@@ -443,19 +454,19 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 
 	/* close pipes */
 	if (ibuf_se) {
-		msgbuf_clear(&ibuf_se->w);
+		imsgbuf_clear(ibuf_se);
 		close(ibuf_se->fd);
 		free(ibuf_se);
 		ibuf_se = NULL;
 	}
 	if (ibuf_rde) {
-		msgbuf_clear(&ibuf_rde->w);
+		imsgbuf_clear(ibuf_rde);
 		close(ibuf_rde->fd);
 		free(ibuf_rde);
 		ibuf_rde = NULL;
 	}
 	if (ibuf_rtr) {
-		msgbuf_clear(&ibuf_rtr->w);
+		imsgbuf_clear(ibuf_rtr);
 		close(ibuf_rtr->fd);
 		free(ibuf_rtr);
 		ibuf_rtr = NULL;
@@ -467,7 +478,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 	pftable_clear_all();
 
 	RB_FOREACH(p, peer_head, &conf->peers)
-		pfkey_remove(p);
+		pfkey_remove(&p->auth_state);
 
 	while ((rr = SIMPLEQ_FIRST(&ribnames)) != NULL) {
 		SIMPLEQ_REMOVE_HEAD(&ribnames, entry);
@@ -651,10 +662,13 @@ send_config(struct bgpd_config *conf)
 		if (imsg_compose(ibuf_se, IMSG_RECONF_PEER, p->conf.id, 0, -1,
 		    &p->conf, sizeof(p->conf)) == -1)
 			return (-1);
+		if (pfkey_send_conf(ibuf_se, p->conf.id, &p->auth_conf) == -1)
+			return (-1);
 
 		if (p->reconf_action == RECONF_REINIT)
-			if (pfkey_establish(p) == -1)
-				log_peer_warnx(&p->conf, "pfkey setup failed");
+			if (pfkey_establish(&p->auth_state, &p->auth_conf,
+			    session_localaddr(p), &p->conf.remote_addr) == -1)
+				log_peer_warnx(&p->conf, "auth setup failed");
 	}
 
 	/* networks go via kroute to the RDE */
@@ -719,14 +733,6 @@ send_config(struct bgpd_config *conf)
 	}
 	free_roatree(&conf->roa);
 	RB_FOREACH(aspa, aspa_tree, &conf->aspa) {
-		/* XXX prevent oversized IMSG for now */
-		if (aspa->num * sizeof(*aspa->tas) >
-		    MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
-			log_warnx("oversized ASPA set for customer-as %s, %s",
-			    log_as(aspa->as), "dropped");
-			continue;
-		}
-
 		if (imsg_compose(ibuf_rtr, IMSG_RECONF_ASPA, 0, 0,
 		    -1, aspa, offsetof(struct aspa_set, tas)) == -1)
 			return (-1);
@@ -943,7 +949,9 @@ dispatch_imsg(struct imsgbuf *imsgbuf, int idx, struct bgpd_config *conf)
 			}
 			p = getpeerbyid(conf, imsg_get_id(&imsg));
 			if (p != NULL) {
-				if (pfkey_establish(p) == -1)
+				if (pfkey_establish(&p->auth_state,
+				    &p->auth_conf, session_localaddr(p),
+				    &p->conf.remote_addr) == -1)
 					log_peer_warnx(&p->conf,
 					    "pfkey setup failed");
 			}
@@ -1048,19 +1056,27 @@ dispatch_imsg(struct imsgbuf *imsgbuf, int idx, struct bgpd_config *conf)
 				reconfpending = 3; /* expecting 2 DONE msg */
 			}
 			break;
-		case IMSG_SOCKET_CONN:
+		case IMSG_SOCKET_SETUP:
 			if (idx != PFD_PIPE_RTR) {
 				log_warnx("connect request not from RTR");
 			} else {
+				uint32_t rtrid = imsg_get_id(&imsg);
 				SIMPLEQ_FOREACH(r, &conf->rtrs, entry) {
-					if (imsg_get_id(&imsg) == r->id)
+					if (rtrid == r->id)
 						break;
 				}
 				if (r == NULL)
-					log_warnx("unknown rtr id %d",
-					    imsg_get_id(&imsg));
+					log_warnx("unknown rtr id %d", rtrid);
 				else
-					bgpd_rtr_connect(r);
+					bgpd_rtr_conn_setup(r);
+			}
+			break;
+		case IMSG_SOCKET_TEARDOWN:
+			if (idx != PFD_PIPE_RTR) {
+				log_warnx("connect request not from RTR");
+			} else {
+				uint32_t rtrid = imsg_get_id(&imsg);
+				bgpd_rtr_conn_teardown(rtrid);
 			}
 			break;
 		case IMSG_CTL_SHOW_RTR:
@@ -1243,7 +1259,7 @@ set_pollfd(struct pollfd *pfd, struct imsgbuf *i)
 	}
 	pfd->fd = i->fd;
 	pfd->events = POLLIN;
-	if (i->w.queued > 0)
+	if (imsgbuf_queuelen(i) > 0)
 		pfd->events |= POLLOUT;
 }
 
@@ -1256,7 +1272,7 @@ handle_pollfd(struct pollfd *pfd, struct imsgbuf *i)
 		return (0);
 
 	if (pfd->revents & POLLOUT)
-		if (msgbuf_write(&i->w) <= 0 && errno != EAGAIN) {
+		if (imsgbuf_write(i) == -1) {
 			log_warn("imsg write error");
 			close(i->fd);
 			i->fd = -1;
@@ -1264,7 +1280,7 @@ handle_pollfd(struct pollfd *pfd, struct imsgbuf *i)
 		}
 
 	if (pfd->revents & POLLIN) {
-		if ((n = imsg_read(i)) == -1 && errno != EAGAIN) {
+		if ((n = imsgbuf_read(i)) == -1) {
 			log_warn("imsg read error");
 			close(i->fd);
 			i->fd = -1;
@@ -1290,29 +1306,23 @@ getsockpair(int pipe[2])
 		fatal("socketpair");
 
 	for (i = 0; i < 2; i++) {
-		for (bsize = MAX_SOCK_BUF; bsize >= 16 * 1024; bsize /= 2) {
-			if (setsockopt(pipe[i], SOL_SOCKET, SO_RCVBUF,
-			    &bsize, sizeof(bsize)) == -1) {
-				if (errno != ENOBUFS)
-					fatal("setsockopt(SO_RCVBUF, %d)",
-					    bsize);
-				log_warn("setsockopt(SO_RCVBUF, %d)", bsize);
-				continue;
-			}
-			break;
+		bsize = MAX_SOCK_BUF;
+		if (setsockopt(pipe[i], SOL_SOCKET, SO_RCVBUF,
+		    &bsize, sizeof(bsize)) == -1) {
+			if (errno != ENOBUFS)
+				fatal("setsockopt(SO_RCVBUF, %d)",
+				    bsize);
+			log_warn("setsockopt(SO_RCVBUF, %d)", bsize);
 		}
 	}
 	for (i = 0; i < 2; i++) {
-		for (bsize = MAX_SOCK_BUF; bsize >= 16 * 1024; bsize /= 2) {
-			if (setsockopt(pipe[i], SOL_SOCKET, SO_SNDBUF,
-			    &bsize, sizeof(bsize)) == -1) {
-				if (errno != ENOBUFS)
-					fatal("setsockopt(SO_SNDBUF, %d)",
-					    bsize);
-				log_warn("setsockopt(SO_SNDBUF, %d)", bsize);
-				continue;
-			}
-			break;
+		bsize = MAX_SOCK_BUF;
+		if (setsockopt(pipe[i], SOL_SOCKET, SO_SNDBUF,
+		    &bsize, sizeof(bsize)) == -1) {
+			if (errno != ENOBUFS)
+				fatal("setsockopt(SO_SNDBUF, %d)",
+				    bsize);
+			log_warn("setsockopt(SO_SNDBUF, %d)", bsize);
 		}
 	}
 }
@@ -1353,7 +1363,7 @@ imsg_send_sockets(struct imsgbuf *se, struct imsgbuf *rde, struct imsgbuf *rtr)
 }
 
 void
-bgpd_rtr_connect(struct rtr_config *r)
+bgpd_rtr_conn_setup(struct rtr_config *r)
 {
 	struct connect_elm *ce;
 	struct sockaddr *sa;
@@ -1372,37 +1382,16 @@ bgpd_rtr_connect(struct rtr_config *r)
 		return;
 	}
 
+	if (pfkey_establish(&ce->auth_state, &r->auth,
+	    &r->local_addr, &r->remote_addr) == -1)
+		log_warnx("rtr %s: pfkey setup failed", r->descr);
+
 	ce->id = r->id;
 	ce->fd = socket(aid2af(r->remote_addr.aid),
 	    SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_TCP);
 	if (ce->fd == -1) {
 		log_warn("rtr %s", r->descr);
-		free(ce);
-		return;
-	}
-
-	if ((sa = addr2sa(&r->local_addr, 0, &len)) != NULL) {
-		if (bind(ce->fd, sa, len) == -1) {
-			log_warn("rtr %s: bind to %s", r->descr,
-			    log_addr(&r->local_addr));
-			close(ce->fd);
-			free(ce);
-			return;
-		}
-	}
-
-	sa = addr2sa(&r->remote_addr, r->remote_port, &len);
-	if (connect(ce->fd, sa, len) == -1) {
-		if (errno != EINPROGRESS) {
-			log_warn("rtr %s: connect to %s:%u", r->descr,
-			    log_addr(&r->remote_addr), r->remote_port);
-			close(ce->fd);
-			free(ce);
-			return;
-		}
-		TAILQ_INSERT_TAIL(&connect_queue, ce, entry);
-		connect_cnt++;
-		return;
+		goto fail;
 	}
 
 	switch (r->remote_addr.aid) {
@@ -1428,12 +1417,41 @@ bgpd_rtr_connect(struct rtr_config *r)
 		return;
 	}
 
-	imsg_compose(ibuf_rtr, IMSG_SOCKET_CONN, ce->id, 0, ce->fd, NULL, 0);
+	if (tcp_md5_set(ce->fd, &r->auth, &r->remote_addr) == -1)
+		log_warn("rtr %s: setting md5sig", r->descr);
+
+	if ((sa = addr2sa(&r->local_addr, 0, &len)) != NULL) {
+		if (bind(ce->fd, sa, len) == -1) {
+			log_warn("rtr %s: bind to %s", r->descr,
+			    log_addr(&r->local_addr));
+			goto fail;
+		}
+	}
+
+	sa = addr2sa(&r->remote_addr, r->remote_port, &len);
+	if (connect(ce->fd, sa, len) == -1) {
+		if (errno != EINPROGRESS) {
+			log_warn("rtr %s: connect to %s:%u", r->descr,
+			    log_addr(&r->remote_addr), r->remote_port);
+			goto fail;
+		}
+		TAILQ_INSERT_TAIL(&connect_queue, ce, entry);
+		connect_cnt++;
+		return;
+	}
+
+	imsg_compose(ibuf_rtr, IMSG_SOCKET_SETUP, ce->id, 0, ce->fd, NULL, 0);
+	TAILQ_INSERT_TAIL(&socket_queue, ce, entry);
+	return;
+
+ fail:
+	if (ce->fd != -1)
+		close(ce->fd);
 	free(ce);
 }
 
 void
-bgpd_rtr_connect_done(int fd, struct bgpd_config *conf)
+bgpd_rtr_conn_setup_done(int fd, struct bgpd_config *conf)
 {
 	struct rtr_config *r;
 	struct connect_elm *ce;
@@ -1472,11 +1490,26 @@ bgpd_rtr_connect_done(int fd, struct bgpd_config *conf)
 		goto fail;
 	}
 
-	imsg_compose(ibuf_rtr, IMSG_SOCKET_CONN, ce->id, 0, ce->fd, NULL, 0);
-	free(ce);
+	imsg_compose(ibuf_rtr, IMSG_SOCKET_SETUP, ce->id, 0, ce->fd, NULL, 0);
+	TAILQ_INSERT_TAIL(&socket_queue, ce, entry);
 	return;
 
 fail:
 	close(fd);
 	free(ce);
 }
+
+void
+bgpd_rtr_conn_teardown(uint32_t id)
+{
+	struct connect_elm *ce;
+
+	TAILQ_FOREACH(ce, &socket_queue, entry) {
+		if (ce->id == id) {
+			pfkey_remove(&ce->auth_state);
+			TAILQ_REMOVE(&socket_queue, ce, entry);
+			free(ce);
+			return;
+		}
+	}
+}
