@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.506 2024/12/13 19:21:03 claudio Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.512 2025/02/04 18:16:56 denis Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -25,6 +25,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <net/if.h>
+#include <netinet/if_ether.h>
 
 #include <poll.h>
 #include <stdarg.h>
@@ -69,7 +70,7 @@
 #define	BGPD_FLAG_DECISION_TRANS_AS	0x0200
 #define	BGPD_FLAG_DECISION_MED_ALWAYS	0x0400
 #define	BGPD_FLAG_DECISION_ALL_PATHS	0x0800
-#define	BGPD_FLAG_NO_AS_SET		0x1000
+#define	BGPD_FLAG_PERMIT_AS_SET		0x1000
 
 #define	BGPD_LOG_UPDATES		0x0001
 
@@ -166,12 +167,14 @@ enum reconf_action {
 #define	AFI_UNSPEC	0
 #define	AFI_IPv4	1
 #define	AFI_IPv6	2
+#define	AFI_L2VPN	25
 
 /* Subsequent Address Family Identifier as per RFC 4760 */
 #define	SAFI_NONE		0
 #define	SAFI_UNICAST		1
 #define	SAFI_MULTICAST		2
 #define	SAFI_MPLS		4
+#define	SAFI_EVPN		70	/* RFC 7432 */
 #define	SAFI_MPLSVPN		128
 #define	SAFI_FLOWSPEC		133
 #define	SAFI_VPNFLOWSPEC	134
@@ -192,7 +195,8 @@ extern const struct aid aid_vals[];
 #define	AID_VPN_IPv6	4
 #define	AID_FLOWSPECv4	5
 #define	AID_FLOWSPECv6	6
-#define	AID_MAX		7
+#define	AID_EVPN	7
+#define	AID_MAX		8
 #define	AID_MIN		1	/* skip AID_UNSPEC since that is a dummy */
 
 #define AID_VALS	{					\
@@ -204,14 +208,33 @@ extern const struct aid aid_vals[];
 	{ AFI_IPv6, AF_INET6, SAFI_MPLSVPN, "IPv6 vpn" },	\
 	{ AFI_IPv4, AF_INET, SAFI_FLOWSPEC, "IPv4 flowspec" },	\
 	{ AFI_IPv6, AF_INET6, SAFI_FLOWSPEC, "IPv6 flowspec" },	\
+	{ AFI_L2VPN, AF_UNSPEC, SAFI_EVPN, "EVPN" },		\
 }
 
 #define BGP_MPLS_BOS	0x01
+#define ESI_ADDR_LEN	10
+
+#define EVPN_ROUTE_TYPE_2	0x02
+#define EVPN_ROUTE_TYPE_3	0x03
+#define EVPN_ROUTE_TYPE_5	0x05
+
+struct evpn_addr {
+	union {
+		struct in_addr	v4;
+		struct in6_addr	v6;
+	};
+	uint32_t	ethtag;
+	uint8_t		mac[ETHER_ADDR_LEN];
+	uint8_t		esi[ESI_ADDR_LEN];
+	uint8_t		aid;
+	uint8_t		type;
+};
 
 struct bgpd_addr {
 	union {
 		struct in_addr		v4;
 		struct in6_addr		v6;
+		struct evpn_addr	evpn;
 		/* maximum size for a prefix is 256 bits */
 	};		    /* 128-bit address */
 	uint64_t	rd;		/* route distinguisher for VPN addrs */
@@ -421,17 +444,19 @@ struct capabilities {
 	}	grestart;
 	int8_t	mp[AID_MAX];		/* multiprotocol extensions, RFC 4760 */
 	int8_t	add_path[AID_MAX];	/* ADD_PATH, RFC 7911 */
+	int8_t	ext_nh[AID_MAX];	/* Ext Nexthop Encoding, RFC 8950 */
 	int8_t	refresh;		/* route refresh, RFC 2918 */
 	int8_t	as4byte;		/* 4-byte ASnum, RFC 4893 */
 	int8_t	enhanced_rr;		/* enhanced route refresh, RFC 7313 */
 	int8_t	policy;			/* Open Policy, RFC 9234, 2 = enforce */
-	int8_t	ext_msg;		/* Extended Msg, RFC8654 */
+	int8_t	ext_msg;		/* Extended Msg, RFC 8654 */
 };
 
 enum capa_codes {
 	CAPA_NONE = 0,
 	CAPA_MP = 1,
 	CAPA_REFRESH = 2,
+	CAPA_EXT_NEXTHOP = 5,
 	CAPA_EXT_MSG = 6,
 	CAPA_ROLE = 9,
 	CAPA_RESTART = 64,
@@ -514,7 +539,7 @@ struct peer_config {
 #define PEERFLAG_TRANS_AS	0x01
 #define PEERFLAG_LOG_UPDATES	0x02
 #define PEERFLAG_EVALUATE_ALL	0x04
-#define PEERFLAG_NO_AS_SET	0x08
+#define PEERFLAG_PERMIT_AS_SET	0x08
 
 struct rde_peer_stats {
 	uint64_t			 prefix_rcvd_update;
@@ -1144,6 +1169,7 @@ struct ext_comm_pairs {
 	{ EXT_COMMUNITY_TRANS_IPV4, 0x0b, "vrfri" },		\
 								\
 	{ EXT_COMMUNITY_TRANS_OPAQUE, 0x06, "ort" },		\
+	{ EXT_COMMUNITY_TRANS_OPAQUE, 0x0c, "encap" },		\
 	{ EXT_COMMUNITY_TRANS_OPAQUE, 0x0d, "defgw" },		\
 								\
 	{ EXT_COMMUNITY_NON_TRANS_OPAQUE, EXT_COMMUNITY_SUBTYPE_OVS, "ovs" }, \
@@ -1581,6 +1607,8 @@ time_t			 getmonotime(void);
 /* util.c */
 char		*ibuf_get_string(struct ibuf *, size_t);
 const char	*log_addr(const struct bgpd_addr *);
+const char	*log_evpnaddr(const struct bgpd_addr *, struct sockaddr *,
+		    socklen_t);
 const char	*log_in6addr(const struct in6_addr *);
 const char	*log_sockaddr(struct sockaddr *, socklen_t);
 const char	*log_as(uint32_t);
@@ -1608,6 +1636,7 @@ int		 nlri_get_vpn4(struct ibuf *, struct bgpd_addr *, uint8_t *,
 		    int);
 int		 nlri_get_vpn6(struct ibuf *, struct bgpd_addr *, uint8_t *,
 		    int);
+int		 nlri_get_evpn(struct ibuf *, struct bgpd_addr *, uint8_t *);
 int		 prefix_compare(const struct bgpd_addr *,
 		    const struct bgpd_addr *, int);
 void		 inet4applymask(struct in_addr *, const struct in_addr *, int);
