@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.111 2024/09/04 13:30:10 claudio Exp $ */
+/*	$OpenBSD: config.c,v 1.113 2024/12/13 19:21:03 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -85,6 +85,7 @@ copy_config(struct bgpd_config *to, struct bgpd_config *from)
 	to->short_as = from->short_as;
 	to->holdtime = from->holdtime;
 	to->min_holdtime = from->min_holdtime;
+	to->staletime = from->staletime;
 	to->connectretry = from->connectretry;
 	to->fib_priority = from->fib_priority;
 	to->filtered_in_locrib = from->filtered_in_locrib;
@@ -440,8 +441,8 @@ merge_config(struct bgpd_config *xconf, struct bgpd_config *conf)
 		np = getpeerbyid(conf, p->conf.id);
 		if (np != NULL) {
 			np->reconf_action = RECONF_KEEP;
-			/* copy the auth state since parent uses it */
-			np->auth = p->auth;
+			/* keep the auth state since parent needs it */
+			np->auth_state = p->auth_state;
 
 			RB_REMOVE(peer_head, &xconf->peers, p);
 			free(p);
@@ -467,7 +468,7 @@ free_deleted_peers(struct bgpd_config *conf)
 	RB_FOREACH_SAFE(p, peer_head, &conf->peers, nextp) {
 		if (p->reconf_action == RECONF_DELETE) {
 			/* peer no longer exists, clear pfkey state */
-			pfkey_remove(p);
+			pfkey_remove(&p->auth_state);
 			RB_REMOVE(peer_head, &conf->peers, p);
 			free(p);
 		}
