@@ -1,4 +1,4 @@
-/*	$OpenBSD: printconf.c,v 1.178 2024/12/13 19:21:03 claudio Exp $	*/
+/*	$OpenBSD: printconf.c,v 1.181 2025/01/27 15:22:11 claudio Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -404,8 +404,8 @@ print_mainconf(struct bgpd_config *conf)
 	if (conf->flags & BGPD_FLAG_DECISION_ALL_PATHS)
 		printf("rde evaluate all\n");
 
-	if (conf->flags & BGPD_FLAG_NO_AS_SET)
-		printf("reject as-set yes\n");
+	if (conf->flags & BGPD_FLAG_PERMIT_AS_SET)
+		printf("reject as-set no\n");
 
 	if (conf->log & BGPD_LOG_UPDATES)
 		printf("log updates\n");
@@ -857,12 +857,12 @@ print_peer(struct peer *peer, struct bgpd_config *conf, const char *c)
 			printf("%s\trde evaluate all\n", c);
 	}
 
-	if (conf->flags & BGPD_FLAG_NO_AS_SET) {
-		if (!(p->flags & PEERFLAG_NO_AS_SET))
-			printf("%s\treject as-set no\n", c);
-	} else {
-		if (p->flags & PEERFLAG_NO_AS_SET)
+	if (conf->flags & BGPD_FLAG_PERMIT_AS_SET) {
+		if (!(p->flags & PEERFLAG_PERMIT_AS_SET))
 			printf("%s\treject as-set yes\n", c);
+	} else {
+		if (p->flags & PEERFLAG_PERMIT_AS_SET)
+			printf("%s\treject as-set no\n", c);
 	}
 
 	if (p->flags & PEERFLAG_LOG_UPDATES)
@@ -969,9 +969,14 @@ print_announce(struct peer_config *p, const char *c)
 		printf("%s\tannounce as4byte no\n", c);
 
 	if (p->capabilities.ext_msg == 2)
-		printf("%s\tannounce extended enforce\n", c);
+		printf("%s\tannounce extended message enforce\n", c);
 	else if (p->capabilities.ext_msg == 1)
-		printf("%s\tannounce extended yes\n", c);
+		printf("%s\tannounce extended message yes\n", c);
+
+	if (p->capabilities.ext_nh[AID_INET] == 2)
+		printf("%s\tannounce extended nexthop enforce\n", c);
+	else if (p->capabilities.ext_nh[AID_INET] == 1)
+		printf("%s\tannounce extended nexthop yes\n", c);
 
 	if (p->capabilities.add_path[AID_MIN] & CAPA_AP_RECV_ENFORCE)
 		printf("%s\tannounce add-path recv enforce\n", c);
