@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.h,v 1.309 2024/12/12 20:19:03 claudio Exp $ */
+/*	$OpenBSD: rde.h,v 1.313 2025/01/27 15:22:11 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org> and
@@ -143,6 +143,7 @@ enum attrtypes {
 	ATTR_EXT_COMMUNITIES=16,
 	ATTR_AS4_PATH=17,
 	ATTR_AS4_AGGREGATOR=18,
+	ATTR_PMSI_TUNNEL=22,
 	ATTR_LARGE_COMMUNITIES=32,
 	ATTR_OTC=35,
 	ATTR_FIRST_UNKNOWN,	/* after this all attributes are unknown */
@@ -157,7 +158,7 @@ enum attrtypes {
 /* by default mask the reserved bits and the ext len bit */
 #define ATTR_DEFMASK		(ATTR_RESERVED | ATTR_EXTLEN)
 
-/* default attribute flags for well known attributes */
+/* default attribute flags for well-known attributes */
 #define ATTR_WELL_KNOWN		ATTR_TRANSITIVE
 
 struct attr {
@@ -354,7 +355,9 @@ int		rde_match_peer(struct rde_peer *, struct ctl_neighbor *);
 /* rde_peer.c */
 int		 peer_has_as4byte(struct rde_peer *);
 int		 peer_has_add_path(struct rde_peer *, uint8_t, int);
-int		 peer_accept_no_as_set(struct rde_peer *);
+int		 peer_has_ext_msg(struct rde_peer *);
+int		 peer_has_ext_nexthop(struct rde_peer *, uint8_t);
+int		 peer_permit_as_set(struct rde_peer *);
 void		 peer_init(struct filter_head *);
 void		 peer_shutdown(void);
 void		 peer_foreach(void (*)(struct rde_peer *, void *), void *);
