@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_aspa.c,v 1.4 2023/04/20 15:44:45 claudio Exp $ */
+/*	$OpenBSD: rde_aspa.c,v 1.5 2023/08/16 08:26:35 claudio Exp $ */
 
 /*
  * Copyright (c) 2022 Claudio Jeker <claudio@openbsd.org>
@@ -23,18 +23,16 @@
 #include "bgpd.h"
 #include "rde.h"
 
-#define	UNKNOWN		0x0
-#define	NOT_PROVIDER	0x1
-#define	PROVIDER	0x2
-
-#define	CP(x, y)	(x | (y << 4))
-#define	CP_GET(x, i)	((x >> (i * 4)) & 0xf)
+enum cp_state {
+	UNKNOWN,
+	NOT_PROVIDER,
+	PROVIDER,
+};
 
 struct rde_aspa_set {
 	uint32_t		 as;
 	uint32_t		 num;
 	uint32_t		*pas;
-	uint32_t		*pas_aid;
 	int			 next;
 };
 
@@ -110,11 +108,9 @@ aspa_lookup(struct rde_aspa *ra, uint32_t asnum)
  * Returns UNKNOWN if cas is not in the ra table or the aid is out of range.
  * Returns PROVIDER if pas is registered for cas for the specified aid.
  * Retruns NOT_PROVIDER otherwise.
- * The returned value includes the result for both IPv4 and IPv6 and needs
- * to be looked at with CP_GET.
  * This function is called very frequently and needs to be fast.
  */
-static uint8_t
+static enum cp_state
 aspa_cp_lookup(struct rde_aspa *ra, uint32_t cas, uint32_t pas)
 {
 	struct rde_aspa_set *aspa;
@@ -122,17 +118,17 @@ aspa_cp_lookup(struct rde_aspa *ra, uint32_t cas, uint32_t pas)
 
 	aspa = aspa_lookup(ra, cas);
 	if (aspa == NULL)
-		return CP(UNKNOWN, UNKNOWN);
+		return UNKNOWN;
 
 	if (aspa->num < 16) {
 		for (i = 0; i < aspa->num; i++) {
 			if (aspa->pas[i] == pas)
 				break;
 			if (aspa->pas[i] > pas)
-				return CP(NOT_PROVIDER, NOT_PROVIDER);
+				return NOT_PROVIDER;
 		}
 		if (i == aspa->num)
-			return CP(NOT_PROVIDER, NOT_PROVIDER);
+			return NOT_PROVIDER;
 	} else {
 		uint32_t lim, x;
 		for (i = 0, lim = aspa->num; lim != 0; lim /= 2) {
@@ -150,21 +146,10 @@ aspa_cp_lookup(struct rde_aspa *ra, uint32_t cas, uint32_t pas)
 			}
 		}
 		if (lim == 0)
-			return CP(NOT_PROVIDER, NOT_PROVIDER);
+			return NOT_PROVIDER;
 	}
 
-	if (aspa->pas_aid == NULL)
-		return CP(PROVIDER, PROVIDER);
-	switch (aspa->pas_aid[i / 16] >> ((i % 16) * 2) & 0x3) {
-	case 0x1:
-		return CP(PROVIDER, NOT_PROVIDER);
-	case 0x2:
-		return CP(NOT_PROVIDER, PROVIDER);
-	case 0x3:
-		return CP(PROVIDER, PROVIDER);
-	default:
-		fatalx("impossible state in aspa_cp_lookup");
-	}
+	return PROVIDER;
 }
 
 /*
@@ -184,14 +169,12 @@ static int
 aspa_check_aspath(struct rde_aspa *ra, struct aspath *a, struct aspa_state *s)
 {
 	uint8_t		*seg;
-	int		 afi;
 	uint32_t	 as, prevas = 0;
 	uint16_t	 len, seg_size;
-	uint8_t		 i, r, seg_type, seg_len;
+	uint8_t		 i, seg_type, seg_len;
 
 	/* the neighbor-as itself is by definition valid */
-	s[0].ndown_p = 1;
-	s[1].ndown_p = 1;
+	s->ndown_p = 1;
 
 	/*
 	 * Walk aspath and validate if necessary both up- and down-ramp.
@@ -213,8 +196,7 @@ aspa_check_aspath(struct rde_aspa *ra, struct aspath *a, struct aspa_state *s)
 			if (as == prevas)
 				continue; /* skip prepends */
 
-			s[0].nhops++;
-			s[1].nhops++;
+			s->nhops++;
 			if (prevas == 0) {
 				prevas = as; /* skip left-most AS */
 				continue;
@@ -226,22 +208,19 @@ aspa_check_aspath(struct rde_aspa *ra, struct aspath *a, struct aspa_state *s)
 			 * node and the right-most provider node
 			 * for which all nodes before are valid.
 			 */
-			r = aspa_cp_lookup(ra, prevas, as);
-			for (afi = 0; afi < 2; afi++) {
-				switch (CP_GET(r, afi)) {
-				case UNKNOWN:
-					if (s[afi].ndown_u == 0)
-						s[afi].ndown_u = s[afi].nhops;
-					break;
-				case PROVIDER:
-					if (s[afi].ndown_p + 1 == s[afi].nhops)
-						s[afi].ndown_p = s[afi].nhops;
-					break;
-				case NOT_PROVIDER:
-					if (s[afi].ndown_np == 0)
-						s[afi].ndown_np = s[afi].nhops;
-					break;
-				}
+			switch (aspa_cp_lookup(ra, prevas, as)) {
+			case UNKNOWN:
+				if (s->ndown_u == 0)
+					s->ndown_u = s->nhops;
+				break;
+			case PROVIDER:
+				if (s->ndown_p + 1 == s->nhops)
+					s->ndown_p = s->nhops;
+				break;
+			case NOT_PROVIDER:
+				if (s->ndown_np == 0)
+					s->ndown_np = s->nhops;
+				break;
 			}
 
 			/*
@@ -252,32 +231,27 @@ aspa_check_aspath(struct rde_aspa *ra, struct aspath *a, struct aspa_state *s)
 			 * We recorde the nhops value of prevas,
 			 * that's why the use of nhops - 1.
 			 */
-			r = aspa_cp_lookup(ra, as, prevas);
-			for (afi = 0; afi < 2; afi++) {
-				switch (CP_GET(r, afi)) {
-				case UNKNOWN:
-					s[afi].nup_p = 0;
-					s[afi].nup_u = s[afi].nhops - 1;
-					break;
-				case PROVIDER:
-					if (s[afi].nup_p == 0)
-						s[afi].nup_p = s[afi].nhops - 1;
-					break;
-				case NOT_PROVIDER:
-					s[afi].nup_p = 0;
-					s[afi].nup_np = s[afi].nhops - 1;
-					break;
-				}
+			switch (aspa_cp_lookup(ra, as, prevas)) {
+			case UNKNOWN:
+				s->nup_p = 0;
+				s->nup_u = s->nhops - 1;
+				break;
+			case PROVIDER:
+				if (s->nup_p == 0)
+					s->nup_p = s->nhops - 1;
+				break;
+			case NOT_PROVIDER:
+				s->nup_p = 0;
+				s->nup_np = s->nhops - 1;
+				break;
 			}
 			prevas = as;
 		}
 	}
 
 	/* the source-as itself is by definition valid */
-	if (s[0].nup_p == 0)
-		s[0].nup_p = s[0].nhops;
-	if (s[1].nup_p == 0)
-		s[1].nup_p = s[1].nhops;
+	if (s->nup_p == 0)
+		s->nup_p = s->nhops;
 	return 0;
 }
 
@@ -324,34 +298,33 @@ aspa_check_finalize(struct aspa_state *state, uint8_t *onlyup, uint8_t *downup)
 /*
  * Validate an aspath against the aspa_set *ra.
  * Returns ASPA_VALID if the aspath is valid, ASPA_UNKNOWN if the
- * aspath contains hops with unknown relation and invalid for
+ * aspath contains hops with unknown relation and ASPA_INVALID for
  * empty aspaths, aspath with AS_SET and aspaths that fail validation.
  */
 void
 aspa_validation(struct rde_aspa *ra, struct aspath *a,
     struct rde_aspa_state *vstate)
 {
-	struct aspa_state state[2] = { 0 };
+	struct aspa_state state = { 0 };
 
 	/* no aspa table, evrything is unknown */
 	if (ra == NULL) {
-		memset(vstate, ASPA_UNKNOWN, 4);
+		memset(vstate, ASPA_UNKNOWN, sizeof(*vstate));
 		return;
 	}
 
 	/* empty ASPATHs are always invalid */
 	if (aspath_length(a) == 0) {
-		memset(vstate, ASPA_INVALID, 4);
+		memset(vstate, ASPA_INVALID, sizeof(*vstate));
 		return;
 	}
 
-	if (aspa_check_aspath(ra, a, state) == -1) {
-		memset(vstate, ASPA_INVALID, 4);
+	if (aspa_check_aspath(ra, a, &state) == -1) {
+		memset(vstate, ASPA_INVALID, sizeof(*vstate));
 		return;
 	}
 
-	aspa_check_finalize(state, &vstate->onlyup_v4, &vstate->downup_v4);
-	aspa_check_finalize(state + 1, &vstate->onlyup_v6, &vstate->downup_v6);
+	aspa_check_finalize(&state, &vstate->onlyup, &vstate->downup);
 }
 
 /*
@@ -402,7 +375,7 @@ aspa_table_prep(uint32_t entries, size_t datasize)
  */
 void
 aspa_add_set(struct rde_aspa *ra, uint32_t cas, const uint32_t *pas,
-    uint32_t pascnt, const uint32_t *pas_aid)
+    uint32_t pascnt)
 {
 	struct rde_aspa_set *aspa;
 	uint32_t h, i;
@@ -441,18 +414,6 @@ aspa_add_set(struct rde_aspa *ra, uint32_t cas, const uint32_t *pas,
 	aspa->pas = ra->data + ra->curdata;
 	for (i = 0; i < pascnt; i++)
 		ra->data[ra->curdata++] = pas[i];
-
-	/* nobody in their right mind has per afi specific data */
-	if (pas_aid != NULL) {
-		/* 2 bits per entry rounded to next uint32_t */
-		if (ra->maxdata - ra->curdata <
-		    TAS_AID_SIZE(pascnt) / sizeof(ra->data[0]))
-			fatalx("aspa set data overflow");
-
-		aspa->pas_aid = ra->data + ra->curdata;
-		memcpy(aspa->pas_aid, pas_aid, TAS_AID_SIZE(pascnt));
-		ra->curdata += TAS_AID_SIZE(pascnt) / sizeof(ra->data[0]);
-	}
 }
 
 void
@@ -476,7 +437,7 @@ aspa_table_stats(const struct rde_aspa *ra, struct ctl_show_set *cset)
 }
 
 /*
- * Return true if the two rde_aspa tables are contain the same data.
+ * Return true if the two rde_aspa tables contain the same data.
  */
 int
 aspa_table_equal(const struct rde_aspa *ra, const struct rde_aspa *rb)
