@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr.c,v 1.21 2024/04/09 12:05:07 claudio Exp $ */
+/*	$OpenBSD: rtr.c,v 1.29 2024/12/02 15:13:57 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -143,7 +143,7 @@ aspa_set_entry(struct aspa_set *aspa, uint32_t asnum)
 	}
 
 	num = aspa->num + 1;
-	newtas = recallocarray(aspa->tas, aspa->num, num, sizeof(uint32_t));
+	newtas = reallocarray(aspa->tas, num, sizeof(uint32_t));
 	if (newtas == NULL)
 		fatal("aspa_set merge");
 
@@ -219,7 +219,10 @@ rtr_main(int debug, int verbose)
 
 	if ((ibuf_main = malloc(sizeof(struct imsgbuf))) == NULL)
 		fatal(NULL);
-	imsg_init(ibuf_main, 3);
+	if (imsgbuf_init(ibuf_main, 3) == -1 ||
+	    imsgbuf_set_maxsize(ibuf_main, MAX_BGPD_IMSGSIZE) == -1)
+		fatal(NULL);
+	imsgbuf_allow_fdpass(ibuf_main);
 
 	conf = new_config();
 	log_info("rtr engine ready");
@@ -264,7 +267,7 @@ rtr_main(int debug, int verbose)
 
 		if (handle_pollfd(&pfd[PFD_PIPE_RDE], ibuf_rde) == -1) {
 			log_warnx("RTR: Lost connection to RDE");
-			msgbuf_clear(&ibuf_rde->w);
+			imsgbuf_clear(ibuf_rde);
 			free(ibuf_rde);
 			ibuf_rde = NULL;
 		} else
@@ -290,11 +293,11 @@ rtr_main(int debug, int verbose)
 
 	/* close pipes */
 	if (ibuf_rde) {
-		msgbuf_clear(&ibuf_rde->w);
+		imsgbuf_clear(ibuf_rde);
 		close(ibuf_rde->fd);
 		free(ibuf_rde);
 	}
-	msgbuf_clear(&ibuf_main->w);
+	imsgbuf_clear(ibuf_main);
 	close(ibuf_main->fd);
 	free(ibuf_main);
 
@@ -309,7 +312,7 @@ rtr_dispatch_imsg_parent(struct imsgbuf *imsgbuf)
 	struct imsg		 imsg;
 	struct bgpd_config	 tconf;
 	struct roa		 roa;
-	char			 descr[PEER_DESCR_LEN];
+	struct rtr_config_msg	 rtrconf;
 	struct rtr_session	*rs;
 	uint32_t		 rtrid;
 	int			 n, fd;
@@ -331,22 +334,25 @@ rtr_dispatch_imsg_parent(struct imsgbuf *imsgbuf)
 			if (ibuf_rde) {
 				log_warnx("Unexpected imsg ctl "
 				    "connection to RDE received");
-				msgbuf_clear(&ibuf_rde->w);
+				imsgbuf_clear(ibuf_rde);
 				free(ibuf_rde);
 			}
 			if ((ibuf_rde = malloc(sizeof(struct imsgbuf))) == NULL)
 				fatal(NULL);
-			imsg_init(ibuf_rde, fd);
+			if (imsgbuf_init(ibuf_rde, fd) == -1 ||
+			    imsgbuf_set_maxsize(ibuf_rde, MAX_BGPD_IMSGSIZE) ==
+			    -1)
+				fatal(NULL);
 			break;
-		case IMSG_SOCKET_CONN:
+		case IMSG_SOCKET_SETUP:
 			if ((fd = imsg_get_fd(&imsg)) == -1) {
 				log_warnx("expected to receive imsg fd "
 				    "but didn't receive any");
 				break;
 			}
 			if ((rs = rtr_get(rtrid)) == NULL) {
-				log_warnx("IMSG_SOCKET_CONN: unknown rtr id %d",
-				    rtrid);
+				log_warnx("IMSG_SOCKET_SETUP: "
+				    "unknown rtr id %d", rtrid);
 				close(fd);
 				break;
 			}
@@ -395,13 +401,14 @@ rtr_dispatch_imsg_parent(struct imsgbuf *imsgbuf)
 			aspa = NULL;
 			break;
 		case IMSG_RECONF_RTR_CONFIG:
-			if (imsg_get_data(&imsg, descr, sizeof(descr)) == -1)
+			if (imsg_get_data(&imsg, &rtrconf,
+			    sizeof(rtrconf)) == -1)
 				fatal("imsg_get_data");
 			rs = rtr_get(rtrid);
 			if (rs == NULL)
-				rtr_new(rtrid, descr);
+				rtr_new(rtrid, &rtrconf);
 			else
-				rtr_config_keep(rs);
+				rtr_config_keep(rs, &rtrconf);
 			break;
 		case IMSG_RECONF_DRAIN:
 			imsg_compose(ibuf_main, IMSG_RECONF_DRAIN, 0, 0,
@@ -532,14 +539,6 @@ rtr_recalc(void)
 	RB_FOREACH_REVERSE(aspa, aspa_tree, &at) {
 		struct aspa_set	as = { .as = aspa->as, .num = aspa->num };
 
-		/* XXX prevent oversized IMSG for now */
-		if (aspa->num * sizeof(*aspa->tas) >
-		    MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
-			log_warnx("oversized ASPA set for customer-as %s, %s",
-			    log_as(aspa->as), "dropped");
-			continue;
-		}
-
 		imsg_compose(ibuf_rde, IMSG_RECONF_ASPA, 0, 0, -1,
 		    &as, offsetof(struct aspa_set, tas));
 		imsg_compose(ibuf_rde, IMSG_RECONF_ASPA_TAS, 0, 0, -1,
