@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr_proto.c,v 1.47 2024/11/26 13:59:54 claudio Exp $ */
+/*	$OpenBSD: rtr_proto.c,v 1.49 2025/01/25 07:23:30 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -42,7 +42,7 @@ struct rtr_header {
 	uint32_t	length;
 } __packed;
 
-#define RTR_MAX_PDU_SIZE	49152	/* XXX < IBUF_READ_SIZE */
+#define RTR_MAX_PDU_SIZE	65535
 #define RTR_MAX_PDU_ERROR_SIZE	256
 #define RTR_DEFAULT_REFRESH	3600
 #define RTR_DEFAULT_RETRY	600
@@ -827,8 +827,10 @@ rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 			return -1;
 		}
 		for (i = 0; i < cnt; i++) {
-			if (ibuf_get_n32(pdu, &aspa->tas[i]) == -1)
+			if (ibuf_get_n32(pdu, &aspa->tas[i]) == -1) {
+				free_aspa(aspa);
 				goto badlen;
+			}
 		}
 	}
 
